package com.wrj.driver.server.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.wrj.driver.server.dto.MemberQueryDto;
import com.wrj.driver.server.dto.UserMemberDto;
import com.wrj.driver.server.entity.TdMember;
import com.wrj.driver.server.entity.TdSysUser;
import com.wrj.driver.server.entity.TdSysUserMember;
import com.wrj.driver.server.service.ITdCarService;
import com.wrj.driver.server.service.ITdMemberService;
import com.wrj.driver.server.service.ITdSysUserMemberService;
import com.wrj.driver.server.service.ITdSysUserService;
import com.wrj.driver.server.vo.UserMemberVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 用户会员表; 前端控制器
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@RestController
@RequestMapping("/tdSysUserMember")
@Slf4j
@Api(tags = "用户会员")
public class TdSysUserMemberController {

    @Resource
    private ITdMemberService memberService;
    @Resource
    private ITdSysUserMemberService userMemberService;
    @Resource
    private ITdCarService tdCarService;
    @Resource
    private ITdSysUserService userService;


    /**
     * 查询会员列表
     * */
    @ApiOperation("查询车型列表")
    @GetMapping("/duima/car/list")
    public Map getCarList(){
        Map<String, Object> result = new HashMap<>();
        result.put("code", 200);
        result.put("data", tdCarService.list());
        return result;
    }


    /**
     * 查询会员列表
     * */
    @ApiOperation("查询会员类型列表")
    @GetMapping("/duima/member/list")
    public Map getMemberList(MemberQueryDto memberQueryDto){
        Map<String, Object> result = new HashMap<>();
        result.put("code", 200);
        result.put("data", memberService.queryMember(memberQueryDto));
        return result;
    }

    //查询用户会员列表
    @ApiOperation("查询会员用户列表")
    @GetMapping("/duima/user/member/list")
    public Map getUserMemberList(UserMemberDto userMemberDto){
        Map<String, Object> result = new HashMap<>();
        result.put("code", 200);
        result.put("data", userMemberService.queryUserMemberList(userMemberDto));
        return result;
    }


    //新增会员

    @ApiOperation("新增用户会员")
    @PostMapping("/duima/user/member")
    public Map addUserMember(@RequestBody UserMemberVo userMemberVo){
        Map<String, Object> result = new HashMap<>();

        final LocalDateTime nowTime = LocalDateTime.now();

        //跟进手机号查询用户
        LambdaQueryWrapper<TdSysUser> cond = new LambdaQueryWrapper<TdSysUser>()
                .eq(TdSysUser::getPhone,userMemberVo.getPhone());
        TdSysUser user = userService.getOne(cond);
        if(user == null){
            result.put("code", 500);
            result.put("msg", "用户不存在");
            return result;
        }
        //查询会员信息
        TdMember member = memberService.getById(userMemberVo.getMemberId());
        if (member == null){
            result.put("code", 500);
            result.put("msg", "会员信息不存在");
            return result;
        }


        TdSysUserMember userMember =   new TdSysUserMember()
                .setMemberId(userMemberVo.getMemberId())
                .setUserId(user.getUserId())
                .setCreateTime(nowTime)
                .setUpdateTime(nowTime)
                .setStartDate(LocalDateTime.of(LocalDate.now(), LocalTime.MIN));
        //会员有效期
        if ("1".equals(member.getUnit())){
            //日
            userMember.setEndDate(LocalDateTime.of(LocalDate.now(), LocalTime.MIN).plusDays(member.getDuration()).minusSeconds(1));
        } else if ("2".equals(member.getUnit())){
            //月
            userMember.setEndDate(LocalDateTime.of(LocalDate.now(), LocalTime.MIN).plusMonths(member.getDuration()).minusSeconds(1));
        } else if ("3".equals(member.getUnit())){
            //年
            userMember.setEndDate(LocalDateTime.of(LocalDate.now(), LocalTime.MIN).plusYears(member.getDuration()).minusSeconds(1));
        }
        result.put("code", 200);
        result.put("data", userMemberService.saveOrUpdate(userMember));
        return result;
    }

}
