package com.wrj.driver.server.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.entity.OrderPayInfo;
import com.wrj.driver.server.entity.TdMember;
import com.wrj.driver.server.entity.TdSysUser;
import com.wrj.driver.server.entity.TdUserDistribution;
import com.wrj.driver.server.enums.PayStatus;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.mapper.OrderPayInfoMapper;
import com.wrj.driver.server.mapper.SysDistributionDetailMapper;
import com.wrj.driver.server.mapper.TdMemberMapper;
import com.wrj.driver.server.mapper.TdUserDistributionMapper;
import com.wrj.driver.server.service.IOrderPayInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.wrj.driver.server.service.ITdMemberService;
import com.wrj.driver.server.service.ITdSysUserService;
import com.wrj.driver.server.util.SecurityUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 支付表 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class OrderPayInfoServiceImpl extends ServiceImpl<OrderPayInfoMapper, OrderPayInfo> implements IOrderPayInfoService {

    @Autowired
    private ITdSysUserService userService;

    @Autowired
    private ITdMemberService memberService;

    @Autowired
    private Snowflake snowflake;

    @Autowired
    private SysDistributionDetailMapper sysDistributionDetailMapper;

    @Autowired
    private TdMemberMapper tdMemberMapper;

    @Autowired
    private TdUserDistributionMapper userDistributionMapper;

    @Override
    public OrderPayInfo createOrderPayInfo(BigDecimal money, String userIdStr, String description, String tradeType, String outTradeNo,
                                           String payType, Long detailId, Long tenantId) {
        Long userId = StrUtil.isNotBlank(userIdStr) ? Long.parseLong(userIdStr) : SecurityUtil.getUserId();
        TdSysUser user = userService.getById(userId);
        if (Objects.isNull(user)) {
            log.error("数据异常,用户不存在" + userId);
            throw new BusinessException("数据缺失");
        }
        TdMember member = memberService.getById(Integer.parseInt(outTradeNo));
        if (Objects.isNull(member) || StrUtil.equals(Constants.IS_ACTIVE_FALSE, member.getIsActive())) {
            log.error("数据异常,会员不存在或者已过期" + outTradeNo);
            throw new BusinessException("数据缺失");
        }
        //查询当前租户下的分销id信息
        String distributionId = null;
        List<TdUserDistribution> userDistributions = userDistributionMapper.selectList(new LambdaQueryWrapper<TdUserDistribution>()
                .eq(TdUserDistribution::getUserId, userId)
                .eq(TdUserDistribution::getTenantId, tenantId));
        if (userDistributions != null && !userDistributions.isEmpty()){
            distributionId = userDistributions.get(0).getDistributionId();
        }

        //查询分销配置信息
//        SysDistributionDetail detail = null;
//        if (null != detailId){
//            detail = sysDistributionDetailMapper.selectById(detailId);
//            if (detail.getDiscountedPrice().compareTo(money) != 0) {
//                log.error("数据异常, 优惠信息已失效" + outTradeNo);
//                throw new BusinessException("优惠信息已失效，请刷新页面后重新操作！");
//            }
//        }

        OrderPayInfo orderPayInfo = new OrderPayInfo()
                .setPayId(snowflake.nextId())
                .setPurchaseProductType(tradeType)
                .setPurchaseProductId(outTradeNo)
                .setMoney(money)
                .setPaymentType(Constants.WECHAT_PAY)
                .setPayType(payType)
                .setDescription(user.getUserName() + "-" + member.getMemberName())
                .setPayStatus(PayStatus.NOT.getValue())
                .setDeleteStatus(0)
                .setUserId(userId)
                .setCreateTime(LocalDateTime.now())
                .setUpdateTime(LocalDateTime.now())
                .setDetailId(detailId)
                .setOriginalPrice(member.getPrice())
                .setTenantId(tenantId)
//                .setCommission(null == detail ? null : detail.getCommission())
                .setDistributionId(distributionId);

        this.save(orderPayInfo);
        return orderPayInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean payNotice( OrderPayInfo payInfo, Boolean payResult) {
        log.info("支付回调信息===============> payInfo{} , payResult{}",payInfo,payResult);
        if (Objects.isNull(payInfo)){
            throw new BusinessException("订单信息不存在");
        }
        payInfo.setPayStatus(payResult?PayStatus.SUCCESS.getValue() : PayStatus.FAIL.getValue())
                .setUpdateTime(LocalDateTime.now());

        this.updateById(payInfo);
        return Boolean.TRUE;
    }

}
